#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.horoscope.chart import charts, house
from jhora.horoscope.dhasa.raasi import narayana
""" Also called Lagna Kendradi Raasi Dhasa """
""" This file also finds Karaka Kendraddi Rasi Dasa - See karaka_kendradhi_rasi_dhasa() """
def lagna_kendradhi_rasi_dhasa(dob,tob,place,divisional_chart_factor=1):
    return kendradhi_rasi_dhasa(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
def kendradhi_rasi_dhasa(dob,tob,place,divisional_chart_factor=1,include_antardhasa=True):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, divisional_chart_factor=divisional_chart_factor)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    asc_house = p_to_h[const._ascendant_symbol]
    seventh_house = (asc_house+7-1)%12
    #print("Finding which house",asc_house,seventh_house,'is stronger')
    dhasa_seed_sign = house.stronger_rasi_from_planet_positions(planet_positions, asc_house, seventh_house)
    #print('dhasa_seed_sign',dhasa_seed_sign)
    direction = 0
    if p_to_h[6]==dhasa_seed_sign:
        direction = 1
    elif p_to_h[8]==dhasa_seed_sign:
        direction = -1
    elif dhasa_seed_sign in const.odd_signs:  # Forward
        direction = 1
    elif dhasa_seed_sign in const.even_signs:  # backward
        direction = -1
    ks = sum(house.kendras()[:3],[])
    #print('ks',ks)
    dhasa_progression = [(dhasa_seed_sign+direction*(k-1))%12 for k in ks]
    dhasa_info = []
    start_jd = jd_at_dob
    for dhasa_lord in dhasa_progression:
        dhasa_duration = narayana._dhasa_duration(planet_positions,dhasa_lord)
        if include_antardhasa:
            bhukthis = _antardhasa(dhasa_lord,p_to_h)
            dd = dhasa_duration/12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,dhasa_duration))
            start_jd += dhasa_duration * const.sidereal_year
    # Second cycle
    dhasa_start = start_jd
    total_dhasa_duration = sum([row[-1] for row in dhasa_info ])
    for c,dhasa_lord in enumerate(dhasa_progression):
        dhasa_duration = 12 - dhasa_info[c][-1]
        total_dhasa_duration += dhasa_duration
        if dhasa_duration <=0: # no need for second cycle as first cycle had 12 years
            continue
        if include_antardhasa:
            bhukthis = _antardhasa(dhasa_lord,p_to_h)
            dd = dhasa_duration/12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,dhasa_duration))
            start_jd += dhasa_duration * const.sidereal_year
        if total_dhasa_duration >= const.human_life_span_for_narayana_dhasa:
            break
    return dhasa_info
def karaka_kendradhi_rasi_dhasa(dob,tob,place,divisional_chart_factor=1,karaka_index=1,include_antardhasa=True):
    if karaka_index not in range(1,9):
        print('Karaka Index should be in the range (1..8). Index 1 assumed')
        karaka_index = 1
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, divisional_chart_factor=divisional_chart_factor)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    ak = house.chara_karakas(planet_positions)[karaka_index-1]
    ak_house = p_to_h[ak]
    seventh_house = (ak_house+7-1)%12
    dhasa_seed_sign = house.stronger_rasi_from_planet_positions(planet_positions, ak_house, seventh_house)
    direction = 0
    if p_to_h[6]==dhasa_seed_sign:
        direction = 1
    elif p_to_h[8]==dhasa_seed_sign:
        direction = -1
    elif dhasa_seed_sign in const.odd_signs:  # Forward
        direction = 1
    elif dhasa_seed_sign in const.even_signs:  # backward
        direction = -1
    ks = sum(house.kendras()[:3],[])
    dhasa_progression = [(dhasa_seed_sign+direction*(k-1))%12 for k in ks]
    dhasa_info = []
    start_jd = jd_at_dob
    for dhasa_lord in dhasa_progression:
        dhasa_duration = narayana._dhasa_duration(planet_positions,dhasa_lord)
        if include_antardhasa:
            bhukthis = _antardhasa(dhasa_lord,p_to_h)
            dd = dhasa_duration/12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,dhasa_duration))
            start_jd += dhasa_duration * const.sidereal_year
    # Second cycle
    dhasa_start = start_jd
    total_dhasa_duration = sum([row[-1] for row in dhasa_info ])
    for c,dhasa_lord in enumerate(dhasa_progression):
        dhasa_duration = 12 - dhasa_info[c][-1]
        total_dhasa_duration += dhasa_duration
        if dhasa_duration <=0: # no need for second cycle as first cycle had 12 years
            continue
        if include_antardhasa:
            bhukthis = _antardhasa(dhasa_lord,p_to_h)
            dd = dhasa_duration/12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,dhasa_duration))
            start_jd += dhasa_duration * const.sidereal_year
        if total_dhasa_duration >= const.human_life_span_for_narayana_dhasa:
            break
    return dhasa_info
    
def _antardhasa(antardhasa_seed_rasi,p_to_h):
    direction = -1
    if p_to_h[6]==antardhasa_seed_rasi or antardhasa_seed_rasi in const.odd_signs: # Forward
        direction = 1
    if p_to_h[8]==antardhasa_seed_rasi:
        direction *= -1
    return [(antardhasa_seed_rasi+direction*i)%12 for i in range(12)]
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.kendradhi_rasi_test()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" Chakra Dhasa """
"""
    Antardasa/Bhukthi Lords do not match with JHora - as it is not clear how it is implemented there
    So we start with mahadasa lord as the bhukthi lord.
"""
from jhora.panchanga import drik
from jhora import utils, const
_dhasa_duration = 10.0; _bhukthi_duration = _dhasa_duration/12.0
year_duration = const.sidereal_year

def _dhasa_seed(jd,place,lagna_house,lagna_lord_house):
    previous_day_sunset_time = drik.sunset(jd-1, place)[0]
    today_sunset_time = drik.sunset(jd, place)[0]
    today_sunrise_time = drik.sunrise(jd, place)[0]
    tomorrow_sunrise_time = 24.0+drik.sunrise(jd+1, place)[0]
    _,_,_,birth_time = utils.jd_to_gregorian(jd)
    df = abs(today_sunset_time - today_sunrise_time)/6.0
    nf1 = abs(today_sunrise_time-previous_day_sunset_time)/6.0
    nf2 = abs(tomorrow_sunrise_time-today_sunset_time)/6.0
    #print('df',df,'nf1',nf1,'nf2',nf2)
    dawn_start = today_sunrise_time-nf1; dawn_end=today_sunrise_time+nf1
    #print('dawn',dawn_start,dawn_end)
    day_start = dawn_end; day_end = today_sunset_time-nf1
    #print('day',day_start,day_end)
    dusk_start = day_end ; dusk_end = today_sunset_time+nf2
    #print('dusk',dusk_start,dusk_end)
    yday_night_start = -(previous_day_sunset_time+nf1); yday_night_end = today_sunrise_time-nf1
    tonight_start = today_sunset_time+nf2; tonight_end = tomorrow_sunrise_time-nf2
    #print('Night-Yday',yday_night_start,yday_night_end,'Night-today',tonight_start,tonight_end)
    # Night is before dawn_start and after dusk_end
    if birth_time > dawn_start and birth_time < dawn_end: # dawn
        kaala_period = 'Dawn'
        _dhasa_seed = (lagna_house+1)%12
    elif birth_time > dusk_start and birth_time < dusk_end: # dusk
        kaala_period = 'Dusk'
        _dhasa_seed = (lagna_house+1)%12
    elif birth_time > day_start and birth_time < day_end: # Day
        kaala_period = 'Day'
        _dhasa_seed = lagna_lord_house
    elif birth_time > yday_night_start and birth_time < yday_night_end: # yday-night
        kaala_period = 'YDay-Night'
        _dhasa_seed = lagna_house
    elif birth_time > tonight_start and birth_time < tonight_end: # yday-night
        kaala_period = 'ToNight'
        _dhasa_seed = lagna_house    
    return _dhasa_seed
def get_dhasa_antardhasa(dob,tob,place,divisional_chart_factor=1,years=1,months=1,sixty_hours=1,include_antardhasa=False):
    jd_at_dob = utils.julian_day_number(dob, tob)
    jd_years = drik.next_solar_date(jd_at_dob, place, years=years, months=months,sixty_hours=months)
    from jhora.horoscope.chart import charts, house
    pp = charts.divisional_chart(jd_years, place, divisional_chart_factor=divisional_chart_factor)
    lagna_house = pp[0][1][0];lagna_lord = house.house_owner_from_planet_positions(pp, lagna_house)
    lagna_lord_house = pp[lagna_lord+1][1][0]
    dhasa_seed = _dhasa_seed(jd_years, place,lagna_house,lagna_lord_house)
    dhasa_info = []
    start_jd = jd_years
    for dhasa_lord in [(dhasa_seed+h)%12 for h in range(12)]:
        _bhukthi_duration = _dhasa_duration/12.0
        if include_antardhasa:
            for bhukthi_lord in [(dhasa_lord+h)%12 for h in range(12)]:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,round(_bhukthi_duration,2)))
                start_jd += _bhukthi_duration * year_duration
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,round(_dhasa_duration,2)))
            start_jd += _dhasa_duration * year_duration
    return dhasa_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.chakra_test()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from PyQt6.QtWidgets import QLineEdit, QApplication, QLabel, QHBoxLayout, QVBoxLayout, QPushButton,\
                            QRadioButton, QDialog, QCheckBox
from PyQt6.QtCore import Qt
from jhora import utils, const
class OptionDialog(QDialog):
    """
        General Options Dialog
    """
    def __init__(self, title='',option_label='', options_list=[],multi_selection=False,default_options=[0]):
        super().__init__()
        self.res = utils.resource_strings
        self._title = 'Options' if title.strip()=='' else title
        self._option_label = 'Select one of the options:' if option_label.strip()=='' else option_label
        self._options_list = options_list
        self._multi_selection = multi_selection
        if not multi_selection and not isinstance(default_options,list): default_options = [default_options]
        self._default_options = default_options
        self.create_ui()
    def create_ui(self):
        v_layout = QVBoxLayout()
        _label = QLabel(self._option_label)
        v_layout.addWidget(_label)
        if len(self._options_list) == 0: return
        self._options = []
        if self._multi_selection:
            for mc in range(len(self._options_list)):
                _caption = self._options_list[mc]
                self._options.append(QCheckBox(_caption))
                if mc in self._default_options: self._options[mc].setChecked(True)
                v_layout.addWidget(self._options[mc])        
        else:
            for mc in range(len(self._options_list)):
                _caption = self._options_list[mc]
                self._options.append(QRadioButton(_caption))
                if mc in self._default_options: self._options[mc].setChecked(True)
                v_layout.addWidget(self._options[mc])        
        h_layout = QHBoxLayout()
        self._accept_button = QPushButton(self.res['accept_str'])
        self._accept_button.clicked.connect(self._accept_button_clicked)
        h_layout.addWidget(self._accept_button)
        self._cancel_button = QPushButton(self.res['cancel_str'])
        self._cancel_button.clicked.connect(self._cancel_button_clicked)
        h_layout.addWidget(self._cancel_button)
        v_layout.addLayout(h_layout)
        self.setLayout(v_layout)
        self.setWindowTitle(self._title)
    def _accept_button_clicked(self):
        self._accept_clicked = True
        if self._multi_selection:
            self._option_string=[];self._option_index=[]
            for mc,rb in enumerate(self._options):
                if rb.isChecked():
                    self._option_string.append(rb.text())
                    self._option_index.append(mc+1)
        else:
            for mc,rb in enumerate(self._options):
                if rb.isChecked():
                    self._option_string = rb.text()
                    self._option_index = mc+1
        self.accept()
    def _cancel_button_clicked(self):
        self._accept_clicked = False
        self._option_string = [] if self._multi_selection else ''
        self._option_index = [] if self._multi_selection else ''
        self.reject()
    def closeEvent(self, *args, **kwargs):
        self._option_string = ''
        QApplication.restoreOverrideCursor()
        return QDialog.closeEvent(self, *args, **kwargs)

if __name__ == "__main__":
    import sys
    utils.set_language('ta')
    def except_hook(cls, exception, traceback):
        print('exception called')
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    App = QApplication(sys.argv)
    chart = OptionDialog(title='Title',option_label='Select options from',options_list=['option1','option2'],
                         multi_selection=False,default_options=1)
    chart.show()
    sys.exit(App.exec())



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" Kaala Dhasa """
from jhora.panchanga import drik
from jhora import utils, const
_kaala_dhasa_life_span = 120 # years

def _dhasa_progression_and_periods(jd,place):
    previous_day_sunset_time = drik.sunset(jd-1, place)[0]
    today_sunset_time = drik.sunset(jd, place)[0]
    today_sunrise_time = drik.sunrise(jd, place)[0]
    tomorrow_sunrise_time = 24.0+drik.sunrise(jd+1, place)[0]
    _,_,_,birth_time = utils.jd_to_gregorian(jd)
    df = abs(today_sunset_time - today_sunrise_time)/6.0
    nf1 = abs(today_sunrise_time-previous_day_sunset_time)/6.0
    nf2 = abs(tomorrow_sunrise_time-today_sunset_time)/6.0
    dawn_start = today_sunrise_time-nf1; dawn_end=today_sunrise_time+nf1
    day_start = dawn_end; day_end = today_sunset_time-nf1
    dusk_start = day_end ; dusk_end = today_sunset_time+nf2
    yday_night_start = -(previous_day_sunset_time+nf1); yday_night_end = today_sunrise_time-nf1
    tonight_start = today_sunset_time+nf2; tonight_end = tomorrow_sunrise_time-nf2
    # Night is before dawn_start and after dusk_end
    if birth_time > dawn_start and birth_time < dawn_end: # dawn
        kaala_type = 0 # 'Dawn'
        kaala_frac = (birth_time-dawn_start)/(dawn_end-dawn_start)
    elif birth_time > dusk_start and birth_time < dusk_end: # dusk
        kaala_type = 2 # 'Dusk'
        kaala_frac = (birth_time-dusk_start)/(dusk_end-dusk_start)
    elif birth_time > day_start and birth_time < day_end: # Day
        kaala_type = 1 # 'Day'
        kaala_frac = (birth_time-day_start)/(day_end-day_start)
    elif birth_time > yday_night_start and birth_time < yday_night_end: # yday-night
        kaala_type = 3 # 'YDay-Night'
        kaala_frac = (birth_time-yday_night_start)/(yday_night_end-yday_night_start)
    elif birth_time > tonight_start and birth_time < tonight_end: # yday-night
        kaala_type = 3 # 'ToNight'
        kaala_frac = (birth_time-tonight_start)/(tonight_end-tonight_start)
    _kaala_dhasa_life_span_first_cycle = _kaala_dhasa_life_span*kaala_frac
    _dhasas1 = [(p+1)*_kaala_dhasa_life_span_first_cycle/45.0 for p in range(9)]
    # Second Cycle
    _kaala_dhasa_life_span_second_cycle = _kaala_dhasa_life_span - _kaala_dhasa_life_span_first_cycle
    _dhasas2 = [(p+1)*_kaala_dhasa_life_span_second_cycle/45.0 for p in range(9)]
    return kaala_type, kaala_frac,_dhasas1,_dhasas2
def get_dhasa_antardhasa(dob,tob,place,years=1,months=1,sixty_hours=1,include_antardhasa=False):
    """
        provides kaala dhasa bhukthi for a given date in julian day (includes birth time)
        @param dob: Date Struct (year,month,day)
        @param tob: time tuple (h,m,s) 
        @param place: Place as tuple (place name, latitude, longitude, timezone)
        @param divisional_chart_factor Default=1 
            1=Raasi, 9=Navamsa. See const.division_chart_factors for options
        @param years: Yearly chart. number of years from date of birth
        @param months: Monthly chart. number of months from date of birth
        @param sixty_hours: 60-hour chart. number of 60 hours from date of birth
        @param include_antardhasa: True (include) False (exclude) antardhasa (Default=True)
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start] if include_antardhasa=True
        @return: a list of [dhasa_lord,dhasa_start] if include_antardhasa=False
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    jd_at_dob = utils.julian_day_number(dob, tob)
    jd_years = drik.next_solar_date(jd_at_dob, place, years=years, months=months,sixty_hours=sixty_hours)
    kaala_type, kaala_frac,dhasas_first, dhasas_second = _dhasa_progression_and_periods(jd_years, place)
    dhasa_info = []
    start_jd = jd_years
    for dhasa_lord in range(9):
        _dhasa_duration = dhasas_first[dhasa_lord]
        if include_antardhasa:
            _dhasa_duration = kaala_frac*dhasas_first[dhasa_lord]
            for bhukthi_lord in range(9):
                _bhukthi_duration = (bhukthi_lord+1)*_dhasa_duration/45.0
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,round(_bhukthi_duration,2)))
                start_jd += _bhukthi_duration * const.sidereal_year
            # Second cycle of Antardhasa
            _dhasa_duration = (1.0-kaala_frac)*dhasas_first[dhasa_lord]
            for bhukthi_lord in range(9):
                _bhukthi_duration = (bhukthi_lord+1)*_dhasa_duration/45.0
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,round(_bhukthi_duration,2)))
                start_jd += _bhukthi_duration * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,round(_dhasa_duration,2)))
            start_jd += _dhasa_duration * const.sidereal_year
    # Second Cycle
    for dhasa_lord in range(9):
        _dhasa_duration = dhasas_second[dhasa_lord]
        if include_antardhasa:
            _dhasa_duration = kaala_frac*dhasas_second[dhasa_lord]
            for bhukthi_lord in range(9):
                _bhukthi_duration = (bhukthi_lord+1)*_dhasa_duration/45.0
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,round(_bhukthi_duration,2)))
                start_jd += _bhukthi_duration * const.sidereal_year
            # Second cycle of Antardhasa
            _dhasa_duration = (1.0-kaala_frac)*dhasas_second[dhasa_lord]
            for bhukthi_lord in range(9):
                _bhukthi_duration = (bhukthi_lord+1)*_dhasa_duration/45.0
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,round(_bhukthi_duration,2)))
                start_jd += _bhukthi_duration * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,round(_dhasa_duration,2)))
            start_jd += _dhasa_duration * const.sidereal_year
    return kaala_type, dhasa_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.kaala_test()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
    This is an attempt to create horoscope based surya sidhantha meant/true position calculations
    Reference: Indian Astronomy - An Introduction - S. Balachandra Rao
    WORK STILL IN PROGRESS - NOT WORKING FOR SOME PLANETS YET
"""
import math
from jhora import utils, const
from jhora.panchanga import drik1 as drik
from jhora.panchanga import surya_sidhantha as ss
import swisseph as swe

if __name__ == "__main__":
    place = drik.Place('Ujjain',23.1765, 75.7885,5.5)
    dt = drik.Date(1997,3,21)
    jd = swe.julday(dt[0],dt[1],dt[2],12.00)
    print('KK Ahargana',ss.ahargana_khanda_khaadyaka(jd))
    print('Graha Laghavam Ahargana',ss.ahargana_graha_laghavam(jd))


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from PyQt6.QtWidgets import QApplication, QLabel, QVBoxLayout, QRadioButton,QDialog, QHBoxLayout,\
                            QPushButton,QSpinBox, QComboBox, QCheckBox, QFrame, QButtonGroup
from jhora import utils, const
from _pytest.mark import param
_chart_names = ['raasi_str', 'hora_str', 'drekkanam_str', 'chaturthamsa_str', 'panchamsa_str', 
    'shashthamsa_str', 'saptamsam_str', 'ashtamsa_str', 'navamsam_str', 'dhasamsam_str', 'rudramsa_str', 
    'dhwadamsam_str', 'shodamsa_str', 'vimsamsa_str', 'chaturvimsamsa_str', 'nakshatramsa_str', 'thrisamsam_str', 
    'khavedamsa_str', 'akshavedamsa_str', 'sashtiamsam_str', 
    'nava_navamsa_str', 'ashtotharamsa_str', 'dwadas_dwadasamsa_str','custom_varga_kundali_str']
_custom_varga_factor = const.DEFAULT_CUSTOM_VARGA_FACTOR
def get_varga_option_dict():
    """ dict: {dcf:(method_count,method_index,base_rasi_index,count_from_end_of_sign)}"""
    _varga_option_dict = {}; _res = utils.resource_strings
    if const.TREAT_STANDARD_CHART_AS_CUSTOM:
        _varga_option_dict[1] = (None,None,None,None)
        for dcf in range(2,const.MAX_DHASAVARGA_FACTOR+1):                
            _opt_count = len([k for k in _res.keys() if 'dn_custom_option' in k ])
            _varga_option_dict[dcf] = (_opt_count,0,None,None)
    else:
        _varga_option_dict[1] = (None,None,None,None)
        for dcf in const.division_chart_factors[1:]:
            _opt_count = len([k for k in _res.keys() if 'd'+str(dcf)+'_option' in k ])
            _varga_option_dict[dcf] = (_opt_count,1,None,None)
        for dcf in [d for d in range(2,const.MAX_DHASAVARGA_FACTOR+1) if d not in const.division_chart_factors]:                
            _opt_count = len([k for k in _res.keys() if 'dn_custom_option' in k ])
            _varga_option_dict[dcf] = (_opt_count,0,None,None)
    return _varga_option_dict
class VargaChartOptionsDialog(QDialog):
    """
        Varga Chart Options Dialog
        @param chart_index 1 for Hora, 2 for drekkana. 
            set chart_index=None if you custom divisional chart and set varga_factor to custom value
        @param varga_factor: None for standard charts and 1..300 for custom chart
        @param chart_method: chart method index (depending number of options available in each chart type)
        @param base_rasi: None for standard charts and custom cyclic charts. 
                For Non-cyclic custom charts - 0 for Aries and 1 for sign itself
        @param count_from_end_of_sign: None standard charts. Trur/False for custom charts 
        @return: 
                _option_string
                _method_index
                _varga_factor
                _base_rasi_index
                _count_from_end_of_sign
    """
    def __init__(self, chart_index=None,chart_method=0,varga_factor=None,base_rasi=None,count_from_end_of_sign=None):
        super().__init__()
        _varga_option_dict = get_varga_option_dict()
        self._custom_chart_index = len(_chart_names)-1
        self.res = utils.resource_strings
        self._base_rasi_index = base_rasi
        if varga_factor==None and (chart_index==None or chart_index < self._custom_chart_index):
            self._varga_factor = const.division_chart_factors[chart_index]
            self._method_count = _varga_option_dict[self._varga_factor][0]
        elif varga_factor != None:
            if varga_factor in const.division_chart_factors:
                self._varga_factor = varga_factor
                self._method_count = _varga_option_dict[self._varga_factor][0]
            else:
                self._varga_factor = varga_factor
                self._method_count = _varga_option_dict[self._varga_factor][0]
        else:
            self._varga_factor = -1
            self._method_count = _varga_option_dict[_custom_varga_factor][0]
        #print(_varga_option_dict[self._varga_factor])
        self._count_from_end_of_sign = count_from_end_of_sign
        self._method_index = chart_method#_varga_option_dict[self._varga_factor][1]
        self._option_string = ''
        self.create_ui()
    def create_ui(self):
        v_layout = QVBoxLayout()
        self._method_rbs = [QRadioButton('') for _ in range(self._method_count)]
        self._varga_option_label = QLabel('')
        radGroup = QButtonGroup(self)
        self._custom_cycle_option = QRadioButton(self.res['dn_custom_cyclic_str'])
        radGroup.addButton(self._custom_cycle_option)
        self._custom_non_cycle_option = QRadioButton(self.res['dn_custom_non_cyclic_str'])
        radGroup.addButton(self._custom_non_cycle_option)
        self._custom_base_combo = QComboBox()
        self._custom_base_combo.addItems(['Aries Always','The rasi itself'])
        self._base_label = QLabel('Select Base Rasi Option:')
        self._custom_count_from_end_of_sign_check = QCheckBox(self.res['dn_custom_count_from_end_of_sign_str'])
        if self._count_from_end_of_sign != None:
            self._custom_count_from_end_of_sign_check.setChecked(self._count_from_end_of_sign)
        if self._varga_factor not in const.division_chart_factors:
            h_layout = QHBoxLayout()
            _cust_label = QLabel(self.res['dn_custom_varga_number_str'])
            h_layout.addWidget(_cust_label)
            self._custom_dvf_combo = QComboBox()#QSpinBox()
            _dvf_list = [str(d) for d in range(2,const.MAX_DHASAVARGA_FACTOR+1) if d not in const.division_chart_factors]
            if const.TREAT_STANDARD_CHART_AS_CUSTOM:
                _dvf_list = [str(d) for d in range(2,const.MAX_DHASAVARGA_FACTOR+1)]
            self._custom_dvf_combo.addItems(_dvf_list)#setRange(2,const.MAX_DHASAVARGA_FACTOR+1)
            #self._custom_dvf_combo.setSingleStep(1)
            if str(self._varga_factor) in _dvf_list:#range(2,const.MAX_DHASAVARGA_FACTOR+1):
                self._custom_dvf_combo.setCurrentText(str(self._varga_factor))
            else:
                self._custom_dvf_combo.setCurrentText(str(const.DEFAULT_CUSTOM_VARGA_FACTOR))
            h_layout.addWidget(self._custom_dvf_combo)
            v_layout.addLayout(h_layout)
            h_layout = QHBoxLayout()
            self._custom_cycle_option.toggled.connect(self._custom_cyclic_changed)
            self._custom_cycle_option.setChecked(self._method_index==0 and self._base_rasi_index==None)
            h_layout.addWidget(self._custom_cycle_option)
            self._custom_non_cycle_option.toggled.connect(self._custom_cyclic_changed)
            h_layout.addWidget(self._custom_non_cycle_option)
            v_layout.addLayout(h_layout)
            h_layout = QHBoxLayout()
            h_layout.addWidget(self._base_label)
            h_layout.addWidget(self._custom_base_combo)
            self._custom_base_combo.setCurrentIndex(0)
            if self._base_rasi_index != None: self._custom_base_combo.setCurrentIndex(self._base_rasi_index)
            v_layout.addLayout(h_layout)
        if self._varga_factor in const.division_chart_factors:
            varga_chart_index = const.division_chart_factors.index(self._varga_factor)
            self._varga_option_label.setText(self.res['varga_option_str']+' '+self.res[_chart_names[varga_chart_index]])
        else:
            self._varga_option_label.setText(self.res['varga_option_str']+' '+self.res[_chart_names[-1]])
        v_layout.addWidget(self._varga_option_label)
        for mc in range(self._method_count):
            if self._varga_factor in const.division_chart_factors:
                rb_caption = self.res['d'+str(self._varga_factor)+'_option'+str(mc+1)+'_str']
            else:
                rb_caption = self.res['dn_custom_option'+str(mc)+'_str']
            self._method_rbs[mc].setText(rb_caption) #.append(QRadioButton(rb_caption))
            if self._varga_factor in const.division_chart_factors and self._method_index==(mc+1): 
                self._method_rbs[mc].setChecked(True)
            if self._varga_factor not in const.division_chart_factors and self._method_index==(mc): 
                self._method_rbs[mc].setChecked(True)
            v_layout.addWidget(self._method_rbs[mc])
        self._option_string = rb_caption
        self._custom_non_cycle_option.setChecked(not self._custom_cycle_option.isChecked())
        self._custom_count_from_end_of_sign_check.setChecked(False)
        if self._count_from_end_of_sign==True: self._custom_count_from_end_of_sign_check.setChecked(True)
        if self._varga_factor not in const.division_chart_factors:
            # Create a horizontal separator
            separator = QFrame()
            separator.setFrameShape(QFrame.Shape.HLine)
            separator.setFrameShadow(QFrame.Shadow.Sunken)
            v_layout.addWidget(separator)
            v_layout.addWidget(self._custom_count_from_end_of_sign_check)
            #self._custom_count_from_end_of_sign_check.setEnabled(False)
        h_layout = QHBoxLayout()
        self._accept_button = QPushButton(self.res['accept_str'])
        self._accept_button.setFlat(False)
        self._accept_button.clicked.connect(self._accept_button_clicked)
        h_layout.addWidget(self._accept_button)
        self._cancel_button = QPushButton(self.res['cancel_str'])
        self._cancel_button.setFlat(False)
        self._cancel_button.clicked.connect(self._cancel_button_clicked)
        h_layout.addWidget(self._cancel_button)
        v_layout.addLayout(h_layout)
        self.setLayout(v_layout)
        self.setWindowTitle(self.res['choose_varga_chart_method_str'])
    def _custom_cyclic_changed(self):
        if self._custom_cycle_option.isChecked():
            self._base_label.setVisible(False)
            self._custom_base_combo.setVisible(False)
            self._varga_option_label.setVisible(False)
            self._custom_count_from_end_of_sign_check.setVisible(False)
            for mc in range(self._method_count):
                self._method_rbs[mc].setVisible(False)
        else:
            self._base_label.setVisible(True)
            self._custom_base_combo.setVisible(True)
            self._varga_option_label.setVisible(True)
            self._custom_count_from_end_of_sign_check.setVisible(True)
            for mc in range(self._method_count):
                self._method_rbs[mc].setVisible(True)
        self.adjustSize()
    def _accept_button_clicked(self):
        self._accept_clicked = True
        for mc,rb in enumerate(self._method_rbs):
            if rb.isChecked():
                self._option_string = rb.text()
                self._method_index = mc+1
                if self._custom_cycle_option.isVisible() and self._custom_cycle_option.isChecked():
                    self._method_index = 0
                elif self._custom_non_cycle_option.isVisible() and self._custom_non_cycle_option.isChecked():
                    self._method_index = mc
        #self._base_rasi_index = None if not self._custom_base_combo.isVisible() else self._custom_base_combo.currentIndex()
        if not self._custom_base_combo.isVisible():
            self._base_rasi_index = None
            bstr = '<br>(Standard) base_rasi=None' if self._varga_factor in const.division_chart_factors else '<br>(Custom-Cyclic) base_rasi=None'
        else:
            self._base_rasi_index = self._custom_base_combo.currentIndex()
            bstr = '<br>Custom Non-Cyclic base_rasi='+ utils.RAASI_LIST[0] if self._base_rasi_index==0 else 'base is the sign'
        self._option_string += bstr# + str(self._base_rasi_index)
        cstr = ' count_from_end_of_sign='
        #self._count_from_end_of_sign = None if not self._custom_count_from_end_of_sign_check.isVisible() else self._custom_count_from_end_of_sign_check.isChecked()
        if not self._custom_count_from_end_of_sign_check.isVisible():
            self._count_from_end_of_sign = None
            cstr += 'None'
        else:
            self._count_from_end_of_sign = self._custom_count_from_end_of_sign_check.isChecked()
            cstr += str(self._count_from_end_of_sign)
        self._option_string += cstr# + str(self._count_from_end_of_sign)
        self._varga_factor = int(self._custom_dvf_combo.currentText()) if self._varga_factor not in const.division_chart_factors else None
        self.accept()
        """
        print('self._method_count',self._method_count,'self._method_index',self._method_index,
              'self._varga_factor',self._varga_factor,'self._option_string',self._option_string,
              '\nself._base_rasi_index',self._base_rasi_index,'self._count_from_end_of_sign',self._count_from_end_of_sign)
        """
    def _cancel_button_clicked(self):
        self._accept_clicked = False
        if self._varga_factor in const.division_chart_factors:
            rb_caption = self.res['d'+str(self._varga_factor)+'_option'+str(self._method_index)+'_str']
        else:
            rb_caption = self.res['dn_custom_option'+str(self._method_index)+'_str']
        self._option_string = rb_caption#self.res['d'+str(self._varga_factor)+'_option'+str(self._method_index)+'_str']
        self.reject()
    
if __name__ == "__main__":
    import sys
    utils.set_language('ta')
    def except_hook(cls, exception, traceback):
        print('exception called')
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    App = QApplication(sys.argv)
    chart = VargaChartOptionsDialog(chart_index=23,chart_method=None)
    #chart = VargaChartOptionsDialog(chart_index=23,chart_method=0)
    #chart = VargaChartOptionsDialog(chart_index=23,varga_factor=63,base_rasi=1,chart_method=5,count_from_end_of_sign=True)
    #chart = VargaChartOptionsDialog(varga_factor=57,chart_method=0,base_rasi=0,count_from_end_of_sign=True)
    #chart = VargaChartOptionsDialog(varga_factor=9,chart_method=3)
    #chart = VargaChartOptionsDialog(chart_index=8,chart_method=3)
    chart.show()
    sys.exit(App.exec())



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const,utils
from jhora.horoscope.chart import charts, house
from jhora.panchanga import drik
year_duration = const.sidereal_year
def _dhasa_duration(planet_positions,sign,varsha_narayana=False):
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    h_to_p = utils.get_house_to_planet_dict_from_planet_to_house_dict(p_to_h)
    lord_of_sign = house.house_owner_from_planet_positions(planet_positions, sign)
    house_of_lord = p_to_h[lord_of_sign]
    dhasa_period = 0
    """ The length of a dasa is determined by the position of the lord of dasa rasi with respect to dasa rasi."""
    dhasa_period = utils.count_rasis(house_of_lord,sign) if sign in const.even_footed_signs \
                            else utils.count_rasis(sign, house_of_lord) 
    dhasa_period -= 1 # Subtract one from the count
    if dhasa_period <=0:
        """
            Exception (1) If the count of houses from dasa rasi to its lord is one, 
            i.e. dasa rasi contains its lord, then we get zero by subtracting one from one. 
            However, dasa length becomes 12 years then.
        """
        dhasa_period = 12
    if const.house_strengths_of_planets[lord_of_sign][house_of_lord] == const._EXALTED_UCCHAM : # > const._FRIEND:
        """ Exception (2) If the lord of dasa rasi is exalted, add one year to dasa length."""
        dhasa_period += 1
    elif const.house_strengths_of_planets[lord_of_sign][house_of_lord] == const._DEFIBILATED_NEECHAM:
        """ Rule (3) If the lord of dasa rasi is debilitated, subtract one year from dasa length."""
        dhasa_period -= 1
    if varsha_narayana:
        dhasa_period *= 3
    return dhasa_period
def _narayana_dhasa_calculation(planet_positions,dhasa_seed_sign,dob,tob,place,years=1, months=1, sixty_hours=1,include_antardhasa=True,varsha_narayana=False):
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    dhasa_factor = year_duration
    if varsha_narayana:
        dhasa_factor /= 360
    dhasa_progression = const.narayana_dhasa_normal_progression[dhasa_seed_sign]
    if p_to_h[8]==dhasa_seed_sign:
        dhasa_progression = const.narayana_dhasa_ketu_exception_progression[dhasa_seed_sign]
    elif p_to_h[6]==dhasa_seed_sign:
        dhasa_progression = const.narayana_dhasa_saturn_exception_progression[dhasa_seed_sign]
    dhasa_periods = []
    jd_at_dob = utils.julian_day_number(dob, tob)
    dhasa_start_jd = drik.next_solar_date(jd_at_dob, place, years=years, months=months, sixty_hours=sixty_hours)
    for dhasa_lord in dhasa_progression:
        dhasa_duration = _dhasa_duration(planet_positions,dhasa_lord,varsha_narayana)
        bhukthis = _narayana_antardhasa(planet_positions,dhasa_lord)#_narayana_antardhasa(dhasa_lord,p_to_h)
        if include_antardhasa:
            dhasa_duration /= 12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(dhasa_start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_periods.append((dhasa_lord,bhukthi_lord,dhasa_start,dhasa_duration))
                dhasa_start_jd += dhasa_duration * dhasa_factor
        else:
            y,m,d,h = utils.jd_to_gregorian(dhasa_start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_periods.append((dhasa_lord,dhasa_start,dhasa_duration))
            dhasa_start_jd += dhasa_duration * dhasa_factor
    # Second cycle
    total_dhasa_duration = sum([row[-1] for row in dhasa_periods ])
    for c,dhasa_lord in enumerate(dhasa_progression):
        dhasa_duration = (12 - dhasa_periods[c][-1])
        total_dhasa_duration += dhasa_duration
        if dhasa_duration <=0: # no need for second cycle as first cycle had 12 years
            continue
        bhukthis = _narayana_antardhasa(planet_positions,dhasa_lord)#_narayana_antardhasa(dhasa_lord,p_to_h)
        if include_antardhasa:
            dhasa_duration /= 12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(dhasa_start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_periods.append((dhasa_lord,bhukthi_lord,dhasa_start,dhasa_duration))
                dhasa_start_jd += dhasa_duration * dhasa_factor
        else:
            y,m,d,h = utils.jd_to_gregorian(dhasa_start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_periods.append((dhasa_lord,dhasa_start,dhasa_duration))
            dhasa_start_jd += dhasa_duration * dhasa_factor
        if varsha_narayana:
            if total_dhasa_duration >= const.human_life_span_for_narayana_dhasa*3:
                break
        else:
            if total_dhasa_duration >= const.human_life_span_for_narayana_dhasa:
                break
    return dhasa_periods
def narayana_dhasa_for_divisional_chart(dob,tob,place,years=1, months=1, sixty_hours=1,divisional_chart_factor=1,include_antardhasa=True):
    if divisional_chart_factor==1:
        return narayana_dhasa_for_rasi_chart(dob, tob, place, years, months, sixty_hours, include_antardhasa)
    # Get Rasi Chart first
    jd_at_dob = utils.julian_day_number(dob,tob)
    planet_positions_rasi = charts.divisional_chart(jd_at_dob, place)
    h_to_p_rasi = utils.get_house_planet_list_from_planet_positions(planet_positions_rasi)
    p_to_h_rasi = utils.get_planet_to_house_dict_from_chart(h_to_p_rasi)
    # For D-n planet_positions_rasi get the lord of nth house in rasi planet_positions_rasi
    seed_house = (p_to_h_rasi[const._ascendant_symbol]+divisional_chart_factor-1)%12
    lord_of_seed_house = house.house_owner_from_planet_positions(planet_positions_rasi, seed_house,check_during_dhasa=True)
    """ 
        Important:
        Take the rasi occupied by Lord of Seed House in the divisional planet_positions_rasi of interest as lagna of varga planet_positions_rasi
    """
    # Get Varga Chart
    varga_planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, divisional_chart_factor=divisional_chart_factor)
    p_to_h_varga = utils.get_planet_house_dictionary_from_planet_positions(varga_planet_positions)
    lord_sign = p_to_h_varga[lord_of_seed_house]
    h_to_p_varga = utils.get_house_planet_list_from_planet_positions(varga_planet_positions)
    h_to_p_varga = utils.get_house_planet_list_from_planet_positions(varga_planet_positions)
    seventh_house = (lord_sign+7-1)%12
    dhasa_seed_sign = house.stronger_rasi_from_planet_positions(varga_planet_positions, lord_sign, seventh_house)
    return _narayana_dhasa_calculation(varga_planet_positions,dhasa_seed_sign,dob,tob,place,years=years, months=months, sixty_hours=sixty_hours,include_antardhasa=include_antardhasa,varsha_narayana=False)
def narayana_dhasa_for_rasi_chart(dob,tob,place,years=1,months=1,sixty_hours=1,include_antardhasa=True):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.rasi_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    p_to_h = utils.get_planet_to_house_dict_from_chart(h_to_p)    
    asc_house = p_to_h[const._ascendant_symbol]
    seventh_house = (asc_house+7-1)%12
    dhasa_seed_sign = house.stronger_rasi_from_planet_positions(planet_positions, asc_house, seventh_house)
    return _narayana_dhasa_calculation(planet_positions,dhasa_seed_sign,dob,tob,place,years=years,months=months,sixty_hours=sixty_hours,include_antardhasa=include_antardhasa,varsha_narayana=False)
def _narayana_antardhasa(planet_positions,dhasa_rasi):
    _DEBUG_ = False
    if _DEBUG_:print('dhasa_rasi',utils.RAASI_LIST[dhasa_rasi])
    lord_of_dhasa_rasi = house.house_owner_from_planet_positions(planet_positions, dhasa_rasi, check_during_dhasa=True)
    if _DEBUG_:print('lord_of_dhasa_rasi',utils.PLANET_NAMES[lord_of_dhasa_rasi])
    house_of_dhasa_rasi_lord = planet_positions[lord_of_dhasa_rasi+1][1][0]
    if _DEBUG_:print('house_of_dhasa_rasi_lord',utils.RAASI_LIST[house_of_dhasa_rasi_lord])
    lord_of_7thhouse_of_dhasa_rasi = house.house_owner_from_planet_positions(planet_positions, (dhasa_rasi+7)%12, check_during_dhasa=True)
    if _DEBUG_:print('lord_of_7thhouse_of_dhasa_rasi',utils.PLANET_NAMES[lord_of_7thhouse_of_dhasa_rasi])
    house_of_dhasa_rasi_lord_7thHouse = planet_positions[lord_of_7thhouse_of_dhasa_rasi+1][1][0]
    if _DEBUG_:print('house_of_dhasa_rasi_lord_7thHouse',utils.RAASI_LIST[house_of_dhasa_rasi_lord_7thHouse])
    antardhasa_seed_rasi = house.stronger_rasi_from_planet_positions(planet_positions, house_of_dhasa_rasi_lord, house_of_dhasa_rasi_lord_7thHouse)
    if _DEBUG_:print('stronger antardhasa_seed_rasi',utils.RAASI_LIST[antardhasa_seed_rasi])
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    return _narayana_antardhasa_old(antardhasa_seed_rasi,p_to_h)
def _narayana_antardhasa_old(antardhasa_seed_rasi,p_to_h):
    direction = -1
    if p_to_h[6]==antardhasa_seed_rasi or antardhasa_seed_rasi in const.odd_signs: # Forward
        direction = 1
    if p_to_h[8]==antardhasa_seed_rasi:
        direction *= -1
    return [(antardhasa_seed_rasi+direction*i)%12 for i in range(12)]
def varsha_narayana_dhasa_bhukthi(dob,tob,place,years=1,months=1,sixty_hours=1,divisional_chart_factor=1,include_antardhasa=True):
    jd_at_dob = utils.julian_day_number(dob, tob)
    jd_at_years = drik.next_solar_date(jd_at_dob, place, years=years)
    rasi_planet_positions = charts.rasi_chart(jd_at_years, place)
    p_to_h_rasi = utils.get_planet_house_dictionary_from_planet_positions(rasi_planet_positions)
    varga_planet_positions = charts.divisional_chart(jd_at_years, place, divisional_chart_factor=divisional_chart_factor)
    p_to_h_varga = utils.get_planet_house_dictionary_from_planet_positions(varga_planet_positions)
    natal_lagna =  p_to_h_rasi[const._ascendant_symbol]
    annual_house = (natal_lagna+(years-1)+divisional_chart_factor-1)%12
    h_to_p_varga = utils.get_house_to_planet_dict_from_planet_to_house_dict(p_to_h_varga)
    annual_house_owner_in_varga = house.house_owner_from_planet_positions(varga_planet_positions,annual_house,check_during_dhasa=True)
    dhasa_seed_sign = p_to_h_varga[annual_house_owner_in_varga]
    nd = _narayana_dhasa_calculation(varga_planet_positions, dhasa_seed_sign, dob,tob,place,years=years,months=months,sixty_hours=sixty_hours,include_antardhasa=include_antardhasa,varsha_narayana=True)
    return nd
    
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.chapter_18_tests()
    pvr_tests.varsha_narayana_tests()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
    unit tests for examples from the book
    Ancient Indian Astronomy - Planetary Positions and Eclipses
    By S. Balachandra Rao, Prof. Dept. of Mathematics, National College, Bangalore
"""
import swisseph as swe
from jhora import utils, const
from jhora.panchanga import drik, surya_sidhantha

place = drik.Place('Ujjain',23.1765, 75.7885,5.5)
dt = drik.Date(1997,3,21)
jd = swe.julday(dt[0],dt[1],dt[2],12.00)
def ss_mean_position_tests():
    chapter = 'Chapter 6.3.1 Examples '
    ka = surya_sidhantha.kali_ahargana(jd)
    print(chapter + 'Kali Ahargana',ka)
    _,sun_mean_long = surya_sidhantha._mean_solar_longitude(jd)
    sun_mean_long1 = surya_sidhantha._planet_mean_longitude(jd, place, const._SUN)
    print(chapter + 'Sun mean longitude - Expected: 334° 14’ 9" - Actual:',sun_mean_long,utils.to_dms(sun_mean_long1,is_lat_long='plong'))
    _,moon_mean_long = surya_sidhantha._mean_lunar_longitude(jd)
    moon_mean_long1 = surya_sidhantha._planet_mean_longitude(jd, place, const._MOON)
    print(chapter + 'Moon mean longitude - Expected: 117° 48’ 25" - Actual:',moon_mean_long,utils.to_dms(moon_mean_long1,is_lat_long='plong'))
    _,moon_apogee_mean_long = surya_sidhantha._mean_lunar_apogee_longitude(jd)
    print(chapter + 'Moon Apogee mean longitude - Expected: 131° 59’ 46" - Actual:',moon_apogee_mean_long)
    _,rahu_mean_long = surya_sidhantha._mean_rahu_longitude(jd)
    print(chapter + 'Rahu mean longitude - Expected: 158° 56’ 29" - Actual:',rahu_mean_long)
    _,ketu_mean_long = surya_sidhantha._mean_ketu_longitude(jd)
    print(chapter + 'Ketu mean longitude - Expected: 338° 56’ 29" - Actual:',ketu_mean_long)
def desantara_correction_tests():
    chapter = 'Chapter 6.3.1 Examples '
    bangalore = drik.Place('Bangalore',12.9716, 77.+35./60.,5.5)
    dc_sun = surya_sidhantha._desantara_correction(bangalore, const._SUN)
    sun_mean_long = surya_sidhantha._planet_mean_longitude(jd, place, const._SUN)
    sun_mean_long_after_dc = sun_mean_long + dc_sun
    print(chapter + 'desantara correction for Sun - Expected 0° 0’ 17.7" - Actual:',utils.to_dms(dc_sun,is_lat_long='plong',round_seconds_to_digits=1))    
    print(chapter + 'sun mean longitude after desantara correction - Expected:334° 13’ 51.3" - Actual:',utils.to_dms(sun_mean_long_after_dc,is_lat_long='plong',round_seconds_to_digits=1))

    dc_moon = surya_sidhantha._desantara_correction(bangalore, const._MOON)
    moon_mean_long = surya_sidhantha._planet_mean_longitude(jd, place, const._MOON)
    moon_mean_long_after_dc = moon_mean_long + dc_moon
    print(chapter + 'desantara correction for Moon - Expected 0° 3’ 57.0" - Actual:',utils.to_dms(dc_moon,is_lat_long='plong',round_seconds_to_digits=1))    
    print(chapter + 'Moon mean longitude after desantara correction - Expected:117° 44’ 28.0" - Actual:',utils.to_dms(moon_mean_long_after_dc,is_lat_long='plong',round_seconds_to_digits=1))

    dc_rahu = surya_sidhantha._desantara_correction(bangalore, const._RAHU)
    rahu_mean_long = surya_sidhantha._planet_mean_longitude(jd, place, const._RAHU)
    rahu_mean_long_after_dc = rahu_mean_long + dc_rahu
    print(chapter + 'desantara correction for Rahu - Expected 0° 0’ 1" - Actual:',utils.to_dms(dc_rahu,is_lat_long='plong'))    
    print(chapter + 'Rahu mean longitude after desantara correction - Expected:158° 56’ 30" - Actual:',utils.to_dms(rahu_mean_long_after_dc,is_lat_long='plong'))
def tamil_date_test():
    dt = drik.Date(2001,7,10)
    jd = swe.julday(dt[0],dt[1],dt[2],23.43)
    place = drik.Place('Ujjain',23.1765, 75.7885,5.5)
    tm,td = drik.tamil_solar_month_and_date(dt, place)
    tithi = drik.tithi(jd, place)
    print('tamil month',tm,'tamil day',td,'tithi',tithi[0])
def ss_true_position_sun_tests():
    chapter = 'Chapter 6.3.1 Examples '
    bangalore = drik.Place('Bangalore',12.9716, 77.+35./60.,5.5)
    dc_sun = surya_sidhantha._desantara_correction(bangalore, const._SUN)
    sun_mean_long = surya_sidhantha._planet_mean_longitude(jd, place, const._SUN)
    sun_mean_long_after_dc = sun_mean_long + dc_sun
    sun_mandaphala = surya_sidhantha._mandaphala_planet(jd, const._SUN, sun_mean_long_after_dc)
    print('Sun Mandaphala Expected:2° 7’ 13" - Actual: ',utils.to_dms(sun_mandaphala,is_lat_long='plong'))
    sun_long_before_bhujantara = sun_mean_long_after_dc+sun_mandaphala
    print('Sun longitude before Bhujantara Expected:336° 21’ 4" - Actual:',utils.to_dms(sun_long_before_bhujantara, is_lat_long='plong'))
    sun_bujantara = surya_sidhantha.bhujantara_correction(const._SUN, sun_mandaphala)
    print('Sun Bhujantara Expected:0° 0’ 21" - Actual: ',utils.to_dms(sun_bujantara,is_lat_long='plong'))
    sun_true_long = surya_sidhantha._planet_true_longitude(jd, place,const._SUN, sun_mean_long_after_dc)
    print('Sun True Longitude after Bhujantara Expected:336° 21’ 04" - Actual:',utils.to_dms(sun_true_long, is_lat_long='plong', round_seconds_to_digits=None))
def ss_true_position_moon_tests():
    chapter = 'Chapter 6.3.1 (ii) Examples '
    bangalore = drik.Place('Bangalore',12.9716, 77.+35./60.,5.5)
    dc_moon = surya_sidhantha._desantara_correction(bangalore, const._MOON)
    moon_mean_long = surya_sidhantha._planet_mean_longitude(jd, bangalore, const._MOON)
    moon_mean_long_after_dc = moon_mean_long + dc_moon
    moon_mandaphala = surya_sidhantha._mandaphala_planet(jd, const._MOON, moon_mean_long_after_dc)
    print('Moon Mandaphala Expected:1° 15’ 3" - Actual: ',utils.to_dms(moon_mandaphala,is_lat_long='plong'))
    moon_long_before_bhujantara = moon_mean_long_after_dc+moon_mandaphala
    print('Moon longitude before Bhujantara Expected:118° 59’ 31" - Actual:',utils.to_dms(moon_long_before_bhujantara, is_lat_long='plong'))
    moon_bujantara = surya_sidhantha.bhujantara_correction(const._MOON, moon_mandaphala)
    print('Moon Bhujantara Expected:0° 5’ 3" - Actual: ',utils.to_dms(moon_bujantara,is_lat_long='plong'))
    moon_true_long = surya_sidhantha._planet_true_longitude(jd,bangalore, const._MOON, moon_mean_long_after_dc)
    print('Moon True Longitude after Bhujantara Expected:119° 4’ 34" - Actual:',utils.to_dms(moon_true_long, is_lat_long='plong', round_seconds_to_digits=None))
def ss_true_position_planet_tests(planet,planet_expected_mean_longitude,planet_expected_true_longitude):
    dt = drik.Date(1991,3,22)
    jd = swe.julday(dt[0],dt[1],dt[2],12.00)
    chapter = 'Chapter 12 Examples '
    bangalore = drik.Place('Bangalore',12.9716, 77.+35./60.,5.5)
    planet_mean_longitude = surya_sidhantha._planet_mean_longitude(jd, place, planet)
    print('Planet:',planet,' Mean Longitude Expected:',planet_expected_mean_longitude,' - Actual:',utils.to_dms(planet_mean_longitude, is_lat_long='plong'))
    planet_true_longitude = surya_sidhantha._planet_true_longitude(jd, place,planet, planet_mean_longitude)
    print('Planet:',planet,' True Longitude Expected:',planet_expected_true_longitude,' - Actual:',utils.to_dms(planet_true_longitude, is_lat_long='plong'))
def ss_planet_mandaphala_tests(planet,planet_mandaphala_expected):
    dt = drik.Date(1991,3,22)
    jd = swe.julday(dt[0],dt[1],dt[2],12.00)
    chapter = 'Chapter 12 Examples '
    bangalore = drik.Place('Bangalore',12.9716, 77.+35./60.,5.5)
    planet_mean_longitude = surya_sidhantha._planet_mean_longitude(jd, place, planet)
    planet_mandaphala = surya_sidhantha._mandaphala_planet(jd, planet, planet_mean_longitude)
    print('Planet:',planet,' Mandaphala Expected:',planet_mandaphala_expected,' - Actual: ',utils.to_dms(planet_mandaphala,is_lat_long='plong'))
        
if __name__ == "__main__":
    ss_mean_position_tests()
    desantara_correction_tests()
    ss_true_position_sun_tests()
    ss_true_position_moon_tests()
    ss_true_position_planet_tests(const._SATURN,'272° 49’ 4"','273° 47’ 45"')
    ss_planet_mandaphala_tests(const._JUPITER,'4° 32’ 59"')


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts
def patyayini_dhasa(jd_years,place,ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1):
    """
        Compute Patyaayini Dhasa
        Should be used for Tajaka Annual charts
        @param jd_years:Julian day number for Tajaka Annual date/time
        @param place: drik.Place struct tuple of ('Place',latitude,longitude,time_zone_offset)
        @param ayanamsa_mode: Default = const._DEFAULT_AYANAMSA_MODE
        @param divisional_chart_factor: Default = 1 (Raasi) - See const.division_chart_factors for other possible values
        @param chart_method: default=1, various methods available for each division chart. See charts module 
        @return patyayini dhasa values as a list [planet, dhasa_duration in days]
        Example: [[5, (1993, 6, 26), 24.9], [3, (1993, 8, 13), 48.1], [1, (1993, 8, 14), 0.57],...]]
    """
    cht = charts.divisional_chart(jd_years,place,ayanamsa_mode,divisional_chart_factor,chart_method=chart_method)
    krisamsas = cht[:-2]  # Exclude Rahu and Ketu
    krisamsas.sort(key=lambda x:x[1][1])
    #for p,(h,long) in krisamsas:
    #    print('krisamsas',p,(h,utils.to_dms(long,is_lat_long='plong')))
    patyamsas = [[p,(h,long-krisamsas[i-1][1][1])] for i,[p,(h,long)] in enumerate(krisamsas) if i>0]
    patyamsas = [krisamsas[0]]+patyamsas
    #print('patyamsas',patyamsas)
    #for p,(h,long) in patyamsas:
    #    print('patyamsas',p,(h,utils.to_dms(long,is_lat_long='plong')))
    patyamsa_sum = sum([long for _,(_,long) in patyamsas])
    _dhasa_period_factors = {p:long/patyamsa_sum for p,(_,long) in patyamsas}
    _dhasa_lords = list(_dhasa_period_factors.keys())
    #print('_dhasa_period_factors',_dhasa_period_factors)
    _dhasas = [[p,const.average_gregorian_year*_dhasa_period_factors[p]] for p,(h,long) in patyamsas]
    #for p,long in _dhasas:
    #    print('_dhasas',p,long)
    jd_start = jd_years
    dhasas = []
    for d,(p,dd) in enumerate(_dhasas):
        #print(d,p,dd)
        bn = d
        db = []
        for b in enumerate(_dhasa_lords):
            pa = _dhasa_lords[bn]
            y,m,d,fh = utils.jd_to_gregorian(jd_start)
            date_str = '%04d-%02d-%02d' %(y,m,d)
            time_str = utils.to_dms(fh,as_string=True)
            bhukthi_start = date_str + ' '+time_str
            db.append([pa,bhukthi_start])
            jd_start += _dhasa_period_factors[pa]*dd
            bn = (bn+1)%len(_dhasa_lords)
        dhasas.append([p,db,dd])
    return dhasas
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.patyayini_tests()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  #
#  PyJHora - Based on the book Vedic Astrology - An Integrated Approach, PVR Narasimha Rao        #
#  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  #
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import setuptools
import os
from types import SimpleNamespace

# load _package_info.py into a SimpleNamespace, without having to import the whole carnatic package
# (this is similar to the way scamp handles things)
package_info_file_path = os.path.join(os.path.dirname(__file__), "./", "_package_info.py")
with open(package_info_file_path, "r") as f:
    file_contents_string = f.read()
package_info_dict: dict = {}
exec(file_contents_string, None, package_info_dict)
package_info = SimpleNamespace(**package_info_dict)


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name=package_info.name,
    version=package_info.version,
    author=package_info.author,
    author_email=package_info.author_email,
    description=package_info.description,
    #include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=package_info.url,
    project_urls=package_info.project_urls,
    packages=setuptools.find_packages(),
    install_requires=package_info.install_requires,
    #extras_require=package_info.extras_require,
    package_data=package_info.package_data,
    classifiers=package_info.classifiers,
)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
    Computation of pindayu, Nisargayu, Amsayu dasa
    Ref: https://medium.com/thoughts-on-jyotish/thoughts-on-the-mathematical-ayur-models-and-their-usage-in-the-dasas-such-as-moola-and-naisargika-517dee1396ae
    NOTE: !!! DO NOT USE THIS YET - NOT FULLY IMPLEMENTED YET !!!
    
"""
from jhora import const, utils
from jhora.horoscope.chart import charts,house
from jhora.panchanga import drik
one_year_days = const.sidereal_year
_TOTAL_PINDAYU = sum(const.pindayu_full_longevity_of_planets)
_TOTAL_NISARGAYU = sum(const.nisargayu_full_longevity_of_planets)
_TOTAL_AMSAYU = 120
_DEBUG = False
"""
    Method = 1 => Santhanam (Ref: Santhanam, Brihat Parasara Hora Shastra)
    Method = 2 => Varahamihira (Ref: https://medium.com/thoughts-on-jyotish/thoughts-on-the-mathematical-ayur-models-and-their-usage-in-the-dasas-such-as-moola-and-naisargika-517dee1396ae)
"""
def _astangata_harana(planet_positions):
    """
    •    When a graha is located near to Surya, this Harana is applied.
    •    This does not apply to Śukra and Śani
    •    1/2 is reduced. This is equivalent to multiplying 1/2 to the Base figure.    
    """
    ignore_planets = [5,6]
    planets_in_combustion = charts.planets_in_combustion(planet_positions)
    if _DEBUG: print('planets_in_combustion',planets_in_combustion)
    planets_in_retrograde = charts.planets_in_retrograde(planet_positions)
    """
    # The superior planets (Mars, Jupiter and Shani) are far from Surya during their retrogression
    #    Add 2,4 to ignore planets if retrograde
    [planets_in_retrograde.remove(sp) for sp in [2,4] if sp in planets_in_retrograde]
    """
    if _DEBUG: print('planets_in_retrograde',planets_in_retrograde)
    _harana_factors = {p:1.0 for p in [const._ascendant_symbol]+[*range(7)]}
    if _DEBUG: print('ignore_planets',ignore_planets)
    temp_dict = {p:0.5 for p in planets_in_combustion if p not in ignore_planets}
    temp_dict1 = {p:0.5 for p in planets_in_retrograde if p not in ignore_planets}
    temp_dict.update(temp_dict1)
    _harana_factors.update(temp_dict)
    return _harana_factors
def _shatru_kshetra_harana(planet_positions,treat_mars_as_strong_in_enemy_sign=True,method=2):
    """
    •    When a Graha occupies its enemy sign, this Harana is applied.
    •    This is not applicable to Vakra graha, which can be interpreted as Retrograde planet or Mangal. 
         It is said that Mangal is strong even in its enemy sign, as it is the god of war.
    •    Retrograde grahas are akin to Exaltation, hence, this weakness does not apply to them.
    •    1/3rd is reduced. This is equivalent to multiplying 2/3 to the Base figure.
    """
    if method==1: treat_mars_as_strong_in_enemy_sign = False
    planets_in_enemy_sign = [p for p,(h,_) in planet_positions[:8] if p!=const._ascendant_symbol and const.house_strengths_of_planets[p][h]==const._ENEMY]
    if treat_mars_as_strong_in_enemy_sign and 2 in planets_in_enemy_sign: planets_in_enemy_sign.remove(2)
    if _DEBUG: print('planets_in_enemy_sign',planets_in_enemy_sign)
    planets_in_retrograde = charts.planets_in_retrograde(planet_positions[:8])
    """
    # The superior planets (Mars, Jupiter and Shani) are far from Surya during their retrogression
    #    Add 2,4 to ignore planets if retrograde
    [planets_in_retrograde.remove(sp) for sp in [2,4] if sp in planets_in_retrograde]
    """
    if _DEBUG: print('planets_in_retrograde',planets_in_retrograde)
    _harana_factors = {p:1.0 for p in [const._ascendant_symbol]+[*range(7)]}
    temp_dict = {p:2/3 for p in planets_in_enemy_sign if p not in planets_in_retrograde}
    _harana_factors.update(temp_dict)
    return _harana_factors
def _shatru_kshetra_harana_santhanam(planet_positions,treat_mars_as_strong_in_enemy_sign=False):
    """
        2. Satru Kshetra Haraoa: If a planet is in its enemy's sign, reduce one third of the basic years and take only two third. 
        An exception is that a retrograde planet, although placed in inimical sign, does not incur this liability. 
        'Vakrachara' is the expression made by Maharishi Parāśara and hence it accepts a retrograde planet. 
        Mars also does lose in an enemy's sign. => treat_mars_as_strong_in_enemy_sign = FALSE
        (This is the main difference between Santhanam(method=1) and Varahamihira (method=2)
        Needless to mention, a planet in neutral’s Sign (or in friend's sign)is not subjected to this reduction.
    """
    return _shatru_kshetra_harana(planet_positions, treat_mars_as_strong_in_enemy_sign=False, method=1)
def _chakrapata_harana(planet_positions,method=2):
    """
    •    When a Graha is located above the horizon (visible hemisphere), this Harana is applied.
    •    The visible Hemisphere is mapped to houses 12,11,10,9,8,7.
    •    Higher reduction is applied when the Graha is closer to the Lagna and lesser reduction is applied 
         when the graha occupies closer to 7th.
    •    The reduction of Shubha graha is 1/2 of that of Krura Grahas. 
         Only Surya, Mangal and Shani are considered Krura here. Kshina Chandra and Budha (ill conjoined)
         Budha are considered Shubha here.
    """
    """
        TODO: Following rules are to be used to determine malefic/benefic planets
        (1) Jupiter and Venus are natural benefics (saumya grahas or subha grahas).
        Mercury becomes a natural benefic when he is alone or with more natural
        benefics. Waxing Moon of Sukla paksha is a natural benefic.
        (2) Sun, Mars, Rahu and Ketu are natural malefics (kroora grahas or paapa grahas).
        Mercury becomes a natural malefic when he is joined by more natural malefics.
        Waning Moon of Krishna paksha is a natural malefic.        
    """
    asc_house = planet_positions[0][1][0]
    subha_asubha_factors = {12:(0,0.5),11:(0.5,0.75),10:(2/3,5/6),9:(3/4,7/8),8:(4/5,9/10),7:(5/6,11/12)}
    _harana_factors = {p:1.0 for p in [const._ascendant_symbol]+[*range(7)]}
    if _DEBUG: print('subha_grahas',subha_grahas,const.natural_benefics,'asubha_grahas',asubha_grahas,const.natural_malefics)
    subha_dict = {p:subha_asubha_factors[(house.get_relative_house_of_planet(asc_house,h))][0] for p,(h,_) in planet_positions if p in subha_grahas and (house.get_relative_house_of_planet(asc_house,h))>6}
    asubha_dict = {p:subha_asubha_factors[((house.get_relative_house_of_planet(asc_house,h)))][1] for p,(h,_) in planet_positions if p in asubha_grahas and (house.get_relative_house_of_planet(asc_house,h))>6}
    _harana_factors.update(subha_dict); _harana_factors.update(asubha_dict)
    return _harana_factors
def _chakrapata_harana_santhanam(planet_positions,bhava_starts_with_ascendant=False): #jd,place,bhava_starts_with_ascendant=False):
    """
    3. Vyayadi Harana: Planets entail reduction if placed anywhere between the 12th and 7th (reckoned in descending order).
        This need not be mistaken to be Drisyardha Hani. Drisyardha means that half of the zodiac which is visible. 
        Hence it is J 8'0° behind the ascendantal cusp, i.e. up to descendant via meridian. 
        Vyayadi Harana figures are: full, half, 1/3, 1/4, 1/5 and l /6th according to the planet being 
        in 12th, 11th, 10th, 9th, 8th and 7th. 
        These are for malefic planets while a benefice in this connection 
        loses only half of what is noted for a malefic. 
        The Moon is ever a benefice for longevity calculations, 
        as per Maharishi Parāśara. 
        Mercury although joining a malefic be treated as a benefice only, for all longevity calculations.)
        
        'C' ÷ ((14 - House) – (DP ÷ BL)) = Loss of Years
            C = Base Aayu
            (DP=Distance of planet from Bhava start; 
            BL=Bhava length and 'House' is the Bhava occupied by the planet under rectification.
    """
    global bhava_houses
    asc_long = planet_positions[0][1][1]
    _harana_factors = {p:1.0 for p in [const._ascendant_symbol]+[*range(7)]}
    #bhava_houses = charts.bhava_houses(jd, place)
    bhava_length = 30.0
    if _DEBUG: print('bhava houses',bhava_houses,bhava_length)
    bhava_start = (asc_long - 15)%30; bhava_end = (asc_long + 15)%30
    if bhava_starts_with_ascendant:
        bhava_start = asc_long; bhava_end = (asc_long + 30)%30
    if _DEBUG: print('bhava start',bhava_start,'bhava end',bhava_end)
    vh = {}
    for p,(h,p_long) in planet_positions[:8]:
        bh = bhava_houses[p]
        if bh > 6:
            dp = (p_long-bhava_start)%30 if p_long > bhava_start else (30-bhava_start+p_long)%30
            if _DEBUG: print('dp',dp)
            vh[p] = 1.0 - (1.0/((14-bh)-(dp/bhava_length)))
    _harana_factors.update(vh)
    return _harana_factors
def _krurodaya_harana_santhanam(planet_positions):
    """
        4. Kroorodaya Harana: Only malefic (i.e. Saturn, the Sm and Mars) entail this check if in the ascendant. Mercury, 
        though joining a natural malefic will not be liable to this reduction. Here the ascendant means the area between 
        the starting and ending points of the sign rising. To find out this reduction, the ascendantal cusp in 
        degrees, minutes and seconds be multiplied by the number of basic years donated by the malefic concerned and 
        divided by 21600. The divider is 21600 as these are the total minutes of arc in the zodiac. 
        The figure so arrived should be reduced from the said malefics basic contribution. 
        However, if a benefice aspects the said malefic, reduce only half of the figure so suggested.
        
        NOTE: In above text from Santhanam, no instructions if multiple malefics in lagna
              Nor it mentions if benefic is closer than malefic ignore reduction
    """
    _malefics = [0,2,6]; _benefics = const.natural_benefics
    _harana_factors = {p:1.0 for p in [const._ascendant_symbol]+[*range(7)]}
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    kh_fraction = 1.0 - (planet_positions[0][1][0]*30+planet_positions[0][1][1])/360.0
    kh1 = {m:kh_fraction for m in _malefics if p_to_h[m]==p_to_h[const._ascendant_symbol]}
    mps = {p:planet_positions[p+1][1][1] for p in asubha_grahas if p_to_h[p]==p_to_h[const._ascendant_symbol]}
    mps = sorted(mps.items(),key=lambda x:x[1]-planet_positions[0][1][1]) # sort by ascending close to lagna
    if len(mps)==0: return _harana_factors
    mp = mps[0][0] # consider the closest one to lagna
    aps = [sp for sp in subha_grahas if p_to_h[sp]==p_to_h[mp] and planet_positions[sp][1][1] < planet_positions[mp][1][1] ]
    if len(aps) > 0: return _harana_factors
    if any([p in _benefics for p in house.aspected_planets_of_the_planet(h_to_p, mp)]):
        kh1.update({mp:0.5*kh_fraction})
    _harana_factors.update(kh1)
    return _harana_factors
def _krurodaya_harana(planet_positions,method=2):
    """
        Reduce by Sum of Graha Aayu of all the Grahas arrived at previous steps * Lagna Longitude in the Rasi / 360º
        For Krurodaya Harana, firstly the Lagna fraction needs to be found, which is to be applied to the sum of all 
            the Graha Aayus to determine the reduction that needs to be applied.
    •    This Harana is applied only when a Krura Graha viz., Surya, Shani and Shukra is rising with the Lagna.
    •    If more than one Krura Graha rises in the Lagna, then the one occupying closer to the Lagna degree is considered 
        for this Harana. The remaining are ignored.
    •    If a Shubha graha viz., Guru, Shukra, Budha and Chandra occupy or aspect the Lagna, then the Harana is halved.
    •    If a Shubha Graha also rises with the Krura Graha in the Lagna, then the Harana is ignore, provided the Shubha Graha is closer to the Lagna degree.
         
    """
    """ TODO: To implement above bullet points """
    if method==1: return _krurodaya_harana_santhanam(planet_positions)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    _harana_factors = {p:1.0 for p in [const._ascendant_symbol]+[*range(7)]}
    kh_fraction = 1.0 - (planet_positions[0][1][0]*30+planet_positions[0][1][1])/360.0
    kh1 = {p:kh_fraction for p in asubha_grahas if p_to_h[p]==p_to_h[const._ascendant_symbol]}
    #print('kh1',kh1)
    """ If more than one Krura Graha rises in the Lagna, then the one occupying closer to the Lagna degree is considered 
        for this Harana. The remaining are ignored."""
    mps = {p:planet_positions[p+1][1][1] for p in asubha_grahas if p_to_h[p]==p_to_h[const._ascendant_symbol]}
    #print('mps',mps,planet_positions[0][1][1])
    mps = sorted(mps.items(),key=lambda x:x[1]-planet_positions[0][1][1]) # sort by ascending close to lagna
    #print('mps',mps)
    if len(mps)==0: return _harana_factors
    mp = mps[0][0] # consider the closest one to lagna
    aps = [sp for sp in subha_grahas if p_to_h[sp]==p_to_h[mp] and planet_positions[sp][1][1] < planet_positions[mp][1][1] ]
    """If a Shubha Graha also rises with the Krura Graha in the Lagna, then the Harana is ignore, 
        provided the Shubha Graha is closer to the Lagna degree. """
    if len(aps) > 0: return _harana_factors
    """ If a Shubha graha viz., Guru, Shukra, Budha and Chandra occupy or aspect the Lagna, then the Harana is halved."""
    if any([p in subha_grahas for p in house.aspected_planets_of_the_planet(h_to_p, mp)]):
        kh1.update({mp:0.5*kh_fraction})
    _harana_factors.update(kh1)
    #print('_krurodaya_harana',_harana_factors)
    return _harana_factors
def _bharana(planet_positions):
    """ This is only needed for Amsayu """
    """
    The Bharanas (Increase in the Base Longevity)
        1.    When the Graha is Retrograde, Exalted or in Svakshetra, then multiply by 3.
        2.    When the Graha is in Sva-Navamsa, Sva-Drekkana or in Vargottama Navamsa, then multiply by 2.
        3.    If a multiplication by both 3 and 2 is applicable to a graha, the higher multiplication factor is applied.
    """
    _bharana_factors = {p:1.0 for p in [const._ascendant_symbol]+[*range(7)]}
    retrograde_planets = charts.planets_in_retrograde(planet_positions)
    pp_3 = charts.drekkana_chart(planet_positions); pp_9 = charts.navamsa_chart(planet_positions)
    chk1 = lambda p:p in retrograde_planets or const.house_strengths_of_planets[p][planet_positions[p+1][1][0]] in [const._EXALTED_UCCHAM, const._OWNER_RULER]
    ph = {p:3.0 for p in range(7) if chk1(p)}; _bharana_factors.update(ph)
    chk23 = lambda p: planet_positions[p+1][1][0]==pp_9[p+1][1][0]
    chk21 = lambda p: const.house_strengths_of_planets[p][pp_9[p+1][1][0]]==const._OWNER_RULER
    chk22 = lambda p: const.house_strengths_of_planets[p][pp_3[p+1][1][0]]==const._OWNER_RULER
    ph1 = {p:2.0 for p in range(7) if chk21(p) or chk22(p) or chk23(p) and _bharana_factors[p]!=2.0}
    _bharana_factors.update(ph1)
    if _DEBUG: print('bharana factors',_bharana_factors)
    return _bharana_factors
def _apply_harana(planet_positions,base_aayu,is_amsayu=False,method=2):
        """ TODO: For AMSAYU there are following special rules
            The base aayu arrived at the previous step needs to increased (Bharana) based on occupation in exaltation, 
            retrogression, Svakshetra, Vargottamamsa, Sva-navamsa, Sva-Drekkana.
            Apply Bharana on base aayu and then haranas
            The Bharanas (Increase in the Base Longevity)
                1.    When the Graha is Retrograde, Exalted or in Svakshetra, then multiply by 3.
                2.    When the Graha is in Sva-Navamsa, Sva-Drekkana or in Vargottama Navamsa, then multiply by 2.
                3.    If a multiplication by both 3 and 2 is applicable to a graha, the higher multiplication factor is applied.
            The Haranas (Decrease in the Base Longevity)
                1.    The same Haranas as the Pindayu and Nisargayu Methods are also applied here. Only Krurodaya Harana is not applied.
                2.    If more than two Haranas between Shatrukshetra and Astangata Harana is applicable to the graha, then the only the higher reduction is applied. The Chakrapata Harana is not affected by this and need to be carried out independently.
        """
        """ Astangata Harana - combusion factor """
        """ Santhanam Book says only one of the harana which is max to be applied for each planet """
        final_harana = {}
        ah = _astangata_harana(planet_positions) # does not depend on the method
        if _DEBUG: print('_astangata_harana',ah,final_harana)
        """ Shatru Kshetra Harana """
        skh = _shatru_kshetra_harana(planet_positions)
        final_harana.update({p:min(v1,skh[p]) for p,v1 in ah.items()})
        if _DEBUG: print('_shatru_kshetra_harana',skh,final_harana)
        """ Chakrapata Harana """
        ch = _chakrapata_harana(planet_positions) if method==2 else _chakrapata_harana_santhanam(planet_positions)#jd, place)
        final_harana.update({p:min(v1,ch[p]) for p,v1 in final_harana.items()})
        if _DEBUG: print('_chakrapata_harana',ch,final_harana)
        """ Krurodaya Harana Not applicable for Amsayu """
        kh = {p:1.0 for p in [const._ascendant_symbol]+[*range(7)]}
        if not is_amsayu: kh = _krurodaya_harana(planet_positions)
        final_harana.update({p:min(v1,kh[p]) for p,v1 in final_harana.items()})
        if _DEBUG: print('_krurodaya_harana',kh,final_harana)
        """ Graha Aayu = Base Aayu * (1 — Shatrukshetra Harana) * (1 — Astangata Harana) * (1 — Chakrapata Harana) """
        #graha_aayu = {p:base_aayu[p]*ah[p]*skh[p]*ch[p]*kh[p] for p in base_aayu.keys()}
        graha_aayu = {p:base_aayu[p]*final_harana[p] for p in base_aayu.keys()}
        if _DEBUG: print(base_aayu,'graha_ayu after harana',graha_aayu)
        return graha_aayu
def _pindayu_santhanam(planet_positions,apply_haranas=True,method=1):
    planet_base_longevity = {}
    for planet in range(7):
        planet_long = planet_positions[planet+1][1][0]*30+planet_positions[planet+1][1][1]
        if _DEBUG: print('planet',planet,'planet_long',planet_long)
        arc_of_longevity = utils.norm360(360+planet_long - const.planet_deep_exaltation_longitudes[planet])
        if _DEBUG: print('planet',planet,'arc_of_longevity',arc_of_longevity)
        if arc_of_longevity > 180.0:
            planet_base_longevity[planet] = const.pindayu_full_longevity_of_planets[planet]*arc_of_longevity/360.0
        else:
            planet_base_longevity[planet] = const.pindayu_full_longevity_of_planets[planet] - const.pindayu_full_longevity_of_planets[planet]*arc_of_longevity/360.0
        if _DEBUG: print('planet',planet,'planet_base_longevity santhanam',planet_base_longevity[planet])
    if apply_haranas:
        return _apply_harana(planet_positions,planet_base_longevity,method=method)
    else:
        return planet_base_longevity
    
def _pindayu(planet_positions,apply_haranas=True,method=2):
    return _pindayu_santhanam(planet_positions, apply_haranas, method)
    planet_base_longevity = {}
    for planet in range(7):
        planet_long = planet_positions[planet+1][1][0]*30+planet_positions[planet+1][1][1]
        arc_of_longevity = utils.norm360(360+planet_long - const.planet_deep_exaltation_longitudes[planet])
        effective_arc = arc_of_longevity - 180 if arc_of_longevity > 180 else arc_of_longevity
        planet_base_longevity[planet] = const.pindayu_full_longevity_of_planets[planet]*effective_arc/360.0
    if _DEBUG: print('planet_base_longevity',planet_base_longevity)
    if apply_haranas:
        return _apply_harana(planet_positions,planet_base_longevity,method=method)
    else:
        return planet_base_longevity
def _nisargayu_santhanam(planet_positions,apply_haranas=True,method=1):
    planet_base_longevity = {}
    for planet in range(7):
        planet_long = planet_positions[planet+1][1][0]*30+planet_positions[planet+1][1][1]
        if _DEBUG: print('_nisargayu_santhanam','planet_long',planet_long)
        arc_of_longevity = utils.norm360(360+planet_long - const.planet_deep_exaltation_longitudes[planet])
        if _DEBUG: print('_nisargayu_santhanam','arc_of_longevity',arc_of_longevity)
        if arc_of_longevity > 180.0:
            planet_base_longevity[planet] = const.nisargayu_full_longevity_of_planets[planet]*arc_of_longevity/360.0
        else:
            planet_base_longevity[planet] = const.nisargayu_full_longevity_of_planets[planet] - const.nisargayu_full_longevity_of_planets[planet]*arc_of_longevity/360.0
        if _DEBUG: print(planet,'planet_base_longevity santhanam',planet_base_longevity[planet])
    if _DEBUG: print('planet_base_longevity santhanam',planet_base_longevity)
    if apply_haranas:
        return _apply_harana(planet_positions,planet_base_longevity,method=method)
    else:
        return planet_base_longevity
def _nisargayu(planet_positions,apply_haranas=True,method=2):
    return _nisargayu_santhanam(planet_positions, apply_haranas, method)
    if method==1: return _nisargayu_santhanam(planet_positions, apply_haranas, method)
    planet_base_longevity = {}
    for planet in range(7):
        planet_long = planet_positions[planet+1][1][0]*30+planet_positions[planet+1][1][1]
        if _DEBUG: print('_nisargayu','planet_long',planet_long)
        arc_of_longevity = utils.norm360(planet_long - const.planet_deep_exaltation_longitudes[planet])
        if _DEBUG: print('_nisargayu','arc_of_longevity',arc_of_longevity,const.planet_deep_exaltation_longitudes[planet])
        effective_arc = arc_of_longevity - 180 if arc_of_longevity > 180 else arc_of_longevity
        if _DEBUG: print('_nisargayu','effective_arc',effective_arc)
        planet_base_longevity[planet] = const.nisargayu_full_longevity_of_planets[planet]*effective_arc/360.0
        if _DEBUG: print(planet,'planet_base_longevity',planet_base_longevity[planet])
    if _DEBUG: print('planet_base_longevity',planet_base_longevity)
    if apply_haranas:
        return _apply_harana(planet_positions,planet_base_longevity)
    else:
        return planet_base_longevity
    
def _amsayu(planet_positions,apply_haranas=True,method=1):
    planet_base_longevity = {}
    for planet,(h,p_long) in planet_positions[1:8]: #range(7):
        planet_long = h*30+p_long
        planet_base_longevity[planet] = (planet_long*108) % 12
        if method==2:planet_base_longevity[planet] = (planet_long*60/200) %12 # Varhamihira
    if _DEBUG: print('planet_base_longevity',planet_base_longevity)
    if apply_haranas:
        bh = _bharana(planet_positions)
        ah =  _apply_harana(planet_positions,planet_base_longevity,is_amsayu=True)
        graha_aayu = {p:ah[p]*bh[p] for p in ah.keys()}
        return graha_aayu
    else:
        return planet_base_longevity
def _stronger_of_lagna_sun_moon(planet_positions):
    sp = house.stronger_planet_from_planet_positions(planet_positions, 0, 1)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    asc_house = p_to_h[const._ascendant_symbol]; sp_house = p_to_h[sp]
    sr = house.stronger_rasi_from_planet_positions(planet_positions, asc_house, sp_house)
    if sr == asc_house: 
        return const._ascendant_symbol
    else:
        return sp
def _lagna_longevity_santhanam(jd,place):
    pp_rasi = charts.rasi_chart(jd, place)
    asc_rasi = pp_rasi[0][1][0]; asc_rasi_lord = house.house_owner_from_planet_positions(pp_rasi, asc_rasi)
    asc_rasi_long = asc_rasi*30+pp_rasi[0][1][1]
    if _DEBUG: print(asc_rasi,'asc rasi long',asc_rasi_long)
    asc_nava = (asc_rasi+8)%12; asc_navamsa_lord = house.house_owner_from_planet_positions(pp_rasi, asc_nava)
    asc_nava_long = asc_nava*30+(asc_rasi_long//30)
    if _DEBUG: print(asc_nava,'asc nava long',asc_nava_long)
    sp = house.stronger_planet_from_planet_positions(pp_rasi, asc_rasi_lord, asc_navamsa_lord)
    if _DEBUG: print(asc_rasi_lord,asc_navamsa_lord,'stronger is',sp)
    _lagna_aayu = asc_rasi_long/30.0;
    if sp==asc_navamsa_lord:
        _lagna_aayu = asc_nava_long/30.0;
    return _lagna_aayu
def _lagna_longevity(jd,place,divisional_chart_factor=9,chart_method=1):
    pp_chart = charts.rasi_chart(jd, place)
    asc_chart = pp_chart[0][1][0]; asc_chart_lord = house.house_owner_from_planet_positions(pp_chart, asc_chart)
    asc_chart_long = asc_chart*30+pp_chart[0][1][1]
    pp_navamsa = charts.divisional_chart(jd, place, divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
    asc_nava = pp_navamsa[0][1][0]; asc_navamsa_lord = house.house_owner_from_planet_positions(pp_navamsa, asc_nava)
    asc_nava_long = asc_nava*30+pp_navamsa[0][1][1]
    _lagna_aayu = asc_chart_long/30.0;
    if const.house_strengths_of_planets[asc_navamsa_lord][asc_nava] > const.house_strengths_of_planets[asc_chart_lord][asc_chart]:
        _lagna_aayu = asc_nava_long/30.0;
    return _lagna_aayu
def _get_aayur_type(planet_positions):
    return _stronger_of_lagna_sun_moon(planet_positions)
def _get_global_constants(jd,place):
    global subha_grahas, asubha_grahas, bhava_houses
    subha_grahas, asubha_grahas = charts.benefics_and_malefics(jd, place, method=1)#BV Raman's method
    bhava_houses = charts.bhava_houses(jd, place)#; bhava_length = 30.0
    """
    bm1 = drik.bhaava_madhya(jd, place,bhava_method=const.bhaava_madhya_method); bhava_madhya = bm1[:]+[bm1[0]]
    if _DEBUG: print('bhava madhya',[utils.to_dms(bm,is_lat_long='plong') for bm in bhava_madhya])
    bhava_lengths = [(bhava_madhya[i+1]-bhava_madhya[i])%60 for i in range(len(bhava_madhya)-1)]
    bhava_starts = [(bhava_madhya[i]-0.5*bhava_lengths[i])%30 for i in range(12) ]
    """
    return
def get_dhasa_antardhasa(jd,place,aayur_type=None,include_antardhasa=True,apply_haranas=True,dhasa_method=2,
                         divisional_chart_factor=9,chart_method=1):
    """
        provides Aayu dhasa bhukthi for a given date in julian day (includes birth time)
        @param jd: Julian day for birthdate and birth time
        @param place: Place as tuple (place name, latitude, longitude, timezone) 
        @param aayur_type (0=Pindayu, 1=Nisargayu, 2=Amsayu, None=Automatically determine whichever is applicable)
        @param include_antardhasa: True (include) False (exclude) antardhasa (Default=True)
        @param apply_haranas: (True/False) whether to or not to apply haranas (Default=True)
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start] if include_antardhasa=True
        @return: a list of [dhasa_lord,dhasa_start] if include_antardhasa=False
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    planet_positions = charts.rasi_chart(jd, place)
    if _DEBUG: print('planet_positions',planet_positions)
    global subha_grahas, asubha_grahas, bhava_houses
    bhava_houses = charts.bhava_houses(jd, place)
    subha_grahas, asubha_grahas = charts.benefics_and_malefics(jd, place, method=1)#BV Raman's method
    bm1 = drik.bhaava_madhya(jd, place,const.bhaava_madhya_method); bhava_madhya = bm1[:]+[bm1[0]]
    if _DEBUG: print('bhava madhya',[utils.to_dms(bm,is_lat_long='plong') for bm in bhava_madhya])
    bhava_lengths = [(bhava_madhya[i+1]-bhava_madhya[i])%60 for i in range(len(bhava_madhya)-1)]
    bhava_starts = [bhava_madhya[i]-0.5*bhava_lengths[i] for i in range(12) ]
    if _DEBUG: print('bhava length',[utils.to_dms(bm,is_lat_long='plong') for bm in bhava_lengths])
    if _DEBUG: print('bhava start',[utils.to_dms(bs%30,is_lat_long='plong') for bs in bhava_starts])
    _lagna_duration = _lagna_longevity(jd,place,divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
    sp = aayur_type if aayur_type!=None else _get_aayur_type(planet_positions)
    if _DEBUG: print('stronger of lagna/Sun/Moon',sp)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    _dhasa_seed = p_to_h[sp]
    if _DEBUG: print('dhasa seed',_dhasa_seed)
    dhasa_progression = charts.order_planets_from_kendras_of_raasi(planet_positions[:8], _dhasa_seed,include_lagna=True)
    if sp in [0,1,const._ascendant_symbol]:
        dhasa_progression = [sp] + [p for p in dhasa_progression if p!=sp]
    if _DEBUG: print('dhasa progression',dhasa_progression)
    start_jd = jd
    dhasas = []
    
    if sp == 0:
        if _DEBUG: print('Aayu type Pindayu')
        _dhasa_duration = _pindayu(planet_positions,apply_haranas,method=dhasa_method)
        _total_duration = _TOTAL_PINDAYU
        _dhasa_type = 0 # Pindyayu = Sun is the lord
    elif sp == 1:
        if _DEBUG: print('aayu type nisargayu')
        _dhasa_duration = _nisargayu(planet_positions,apply_haranas,method=dhasa_method)
        _total_duration = _TOTAL_NISARGAYU
        _dhasa_type = 1 # Nisargayu Moon is the lord
    else: # sp == const._ascendant_symbol:
        if _DEBUG: print('aayu type amsayu')
        _dhasa_duration = _amsayu(planet_positions,apply_haranas,method=dhasa_method)
        _total_duration = _TOTAL_AMSAYU
        _dhasa_type = 2 # Amsayu - Lagna is the lord
    _dhasa_duration[const._ascendant_symbol] = _lagna_longevity(jd, place)
    if _DEBUG: print('dhasa duration',_dhasa_duration)
    for lord in dhasa_progression:
        dd = _dhasa_duration[lord]
        bhukthis = dhasa_progression # Antardhasa follows same dhasa progression
        if include_antardhasa:
            ddb = dd/(len(dhasa_progression))
            for bhukthi in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasas.append((lord,bhukthi,dhasa_start,round(ddb,2)))
                start_jd += ddb * one_year_days
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasas.append((lord,dhasa_start,round(dd,2)))
            start_jd += dd * one_year_days
    return _dhasa_type, dhasas
def pindayu_dhasa_bhukthi(jd,place,include_antardhasa=True,apply_haranas=True,dhasa_method=2,
                          divisional_chart_factor=9,chart_method=1):
    return get_dhasa_antardhasa(jd, place, aayur_type=0, include_antardhasa=include_antardhasa, 
                                apply_haranas=apply_haranas, dhasa_method=dhasa_method,
                                divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)[1]
def nisargayu_dhasa_bhukthi(jd,place,include_antardhasa=True,apply_haranas=True,dhasa_method=2,
                          divisional_chart_factor=9,chart_method=1):
    return get_dhasa_antardhasa(jd, place, aayur_type=1, include_antardhasa=include_antardhasa, 
                                apply_haranas=apply_haranas, dhasa_method=dhasa_method,
                                divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)[1]
def amsayu_dhasa_bhukthi(jd,place,include_antardhasa=True,apply_haranas=True,dhasa_method=2,
                          divisional_chart_factor=9,chart_method=1):
    return get_dhasa_antardhasa(jd, place, aayur_type=2, include_antardhasa=include_antardhasa, 
                                apply_haranas=apply_haranas, dhasa_method=dhasa_method,
                                divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)[1]
def longevity(jd,place,aayu_type=None,dhasa_method=2):
    _at,_adb = get_dhasa_antardhasa(jd, place, aayur_type=aayu_type, include_antardhasa=False, apply_haranas=True, dhasa_method=dhasa_method)
    _longevity = sum(d for _,_,d in _adb)
    return _longevity,_at
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.aayu_test()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" Computes Drig Dhasa from the chart """
from jhora import const,utils
from jhora.horoscope.chart import house,charts
from jhora.horoscope.dhasa.raasi import narayana
def drig_dhasa_bhukthi(dob,tob,place,divisional_chart_factor=1,include_antardhasa=True):
    jd = utils.julian_day_number(dob,tob)
    planet_positions = charts.divisional_chart(jd, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, divisional_chart_factor=divisional_chart_factor)
    return drig_dhasa(planet_positions, dob,tob,include_antardhasa=include_antardhasa)
def drig_dhasa(planet_positions,dob,tob,include_antardhasa=True):
    """
        computes drig dhasa from the chart
        @param chart: chart list 1-D. Format ['1/2','3/L',...,'',5/6/7','9','0'] # 12 houses with planets and Lagnam
        @param dob: tuple of date of birth format: (year,month,day)
        @return: list of drig dhasa from date of birth 
          Format: [ [dhasa_lord, dhasa_start_date, dhasa_end_date, [bhukthi_lord1, bhukthi_lord2...], dhasa_duration],...]
          Example: [[2, '1912-1-1', '1916-1-1', [2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 0, 1], 4], 
                    [5, '1916-1-1', '1927-1-1', [5, 4, 3, 2, 1, 0, 11, 10, 9, 8, 7, 6], 11], ...]]
    """
    start_jd = utils.julian_day_number(dob, tob)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    p_to_h = utils.get_planet_to_house_dict_from_chart(h_to_p)  
    asc_house = p_to_h[const._ascendant_symbol]
    ninth_house = (asc_house+9-1)%12
    dhasa_progression= []
    for s in range(ninth_house,(ninth_house+3)):
        s %= 12
        aspected_kendras = house.aspected_kendras_of_raasi(s,s in const.even_footed_signs)
        dp = [s]+ aspected_kendras
        #print(s,dp)
        dhasa_progression.append(dp)
    dhasa_progression = sum(dhasa_progression,[])
    dhasa_info = []
    for dhasa_lord in dhasa_progression:
        dhasa_duration = round(narayana._dhasa_duration(planet_positions,dhasa_lord),2)
        if include_antardhasa:
            bhukthis = _antardhasa(dhasa_lord,p_to_h)
            dd = dhasa_duration/12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,dhasa_duration))
            start_jd += dhasa_duration * const.sidereal_year
    # Second cycle
    dhasa_start = start_jd
    total_dhasa_duration = sum([row[-1] for row in dhasa_info ])
    for c,dhasa_lord in enumerate(dhasa_progression):
        dhasa_duration = 12 - dhasa_info[c][-1]
        dhasa_duration = round(dhasa_duration,2)
        total_dhasa_duration += dhasa_duration
        if dhasa_duration <=0: # no need for second cycle as first cycle had 12 years
            continue
        if include_antardhasa:
            bhukthis = _antardhasa(dhasa_lord,p_to_h)
            dd = dhasa_duration/12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,dhasa_duration))
            start_jd += dhasa_duration * const.sidereal_year
        if total_dhasa_duration >= const.human_life_span_for_narayana_dhasa:
            break
    return dhasa_info
def _antardhasa(antardhasa_seed_rasi,p_to_h):
    direction = -1
    if p_to_h[6]==antardhasa_seed_rasi or antardhasa_seed_rasi in const.odd_signs: # Forward
        direction = 1
    if p_to_h[8]==antardhasa_seed_rasi:
        direction *= -1
    return [(antardhasa_seed_rasi+direction*i)%12 for i in range(12)]
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.drig_dhasa_tests()
